#include <stdlib.h>

#include "list.h"

list *
list_new(void)
{
	list *new;

	new = malloc(sizeof(*new));
	if (new != NULL) {
		new->next = NULL;
		new->data = NULL;
	}

	return new;
}

void
list_apply(list *lp, void (*fn)(list *, void *), void *args)
{
	while (lp != NULL) {
		(fn)(lp, args);
		lp = lp->next;
	}
}

list *
list_lookup(list **lp, list *elem, list_cmp cmp, int insert)
{
	list *prev, *curr;

	prev = NULL;
	curr = (*lp);
	while (curr != NULL) {
		if ((cmp)(curr->data, elem->data) == 0) {
			return curr;
		}

		prev = curr;
		curr = curr->next;
	}

	if (insert != 0) {
		if (prev == NULL) {
			*lp = elem;
		} else {
			prev->next = elem;
		}

		return elem;
	}

	return NULL;
}

void
list_del(list **lp, void *data)
{
	list *prev, *curr;

	prev = NULL;
	curr = *lp;

	while (curr != NULL) {
		if (curr->data == data) {
			break;
		}

		prev = curr;
		curr = curr->next;
	}

	if (curr == NULL) {
		return;
	}

	if (prev == NULL) {
		*lp = curr->next;
	} else {
		prev->next = curr->next;
	}

	free(curr);
}

void
list_free(list **lp)
{
	list *next;

	while ((*lp) != NULL) {
		next = (*lp)->next;
		free(*lp);
		(*lp) = next;
	}

	(*lp) = NULL;
}

